/*	DOS Stamp RTC functions
	(C)1999 Bagotronix, Inc.
	Author: Ivan Baggett

	Written in Borland C v3.0.
	For Dallas DS1689 RTC chip
*/

#include <dos.h>

#include "stdinc.h"
#include "dosstamp.h"
#include "dsrtc.h"


/*	Turns on RTC oscillator
	comments:
		RTC cannot keep time unless oscillator is turned on.
*/
void RTCon (void) {
	UBYTE u;

	u = inportb (RTC_REGA);
	u &= ~0x60;					// isolate bits DV2 and DV1
	u |= 0x20;					// DV2 = 0, DV1 = 1, DV0 = don't care
	outportb (RTC_REGA, u);
}


/*	Turns off RTC oscillator
	comments:
		RTC cannot keep time if oscillator is turned off.
		If the DOS Stamp is to be in storage for a long time, turning off
		the oscillator can save some battery life.
*/
void RTCoff (void) {
	UBYTE u;

	u = inportb (RTC_REGA);
	u &= ~0x60;					// DV2 = 0, DV1 = 0, DV0 = don't care
	outportb (RTC_REGA, u);
}

/*	Initializes RTC
	comments:
		Sets data mode (DM) to BCD, which is what BIOS uses.
		Sets 24/12 mode to 24-hour mode, which is what BIOS uses.
		Clears SQWE bit, because SQW pin is not used.
		Does not turn on the oscillator.  Use RTCon() for that.
*/
void RTCinit (void) {
	UBYTE u;

	u = inportb (RTC_REGB);
	u &= ~0x0c;					// clear SQWE bit and DM bit for BCD op
	u |= 0x02;					// set 24/12 bit for 24-hour op
	outportb (RTC_REGB, u);
}


/*	Sets RTC time
	arguments:
		Hour, minute, and second as unsigned binary
	comments:
		Does not set date.  Use RTCsetdate() for that.

*/
void RTCsettime (UWORD hr, UWORD min, UWORD sec) {
	UBYTE bcdhr, bcdmin, bcdsec, u;


	bcdhr = ubtobcd (hr);
	bcdmin = ubtobcd (min);
	bcdsec = ubtobcd (sec);

	// wait for UIP bit == 0
	while (inportb (RTC_REGA) & 0x80);

	// set SET bit to inhibit update transfers
	u = inportb (RTC_REGB);
	u |= 0x80;
	outportb (RTC_REGB, u);

	outportb (RTC_SEC, bcdsec);
	outportb (RTC_MIN, bcdmin);
	outportb (RTC_HR, bcdhr);

	// clear SET bit to allow update transfers
	u &= ~0x80;
	outportb (RTC_REGB, u);
}


/*	Gets RTC time
	arguments:
		Pointers to hour, minute, and second as unsigned binary
	comments:
		Does not get date.  Use RTCgetdate() for that.

*/
void RTCgettime (UWORD *hr, UWORD *min, UWORD *sec) {
	UBYTE bcdhr, bcdmin, bcdsec;

	// wait for UIP bit == 0
	while (inportb (RTC_REGA) & 0x80);

	bcdsec = inportb (RTC_SEC);
	bcdmin = inportb (RTC_MIN);
	bcdhr = inportb (RTC_HR);

	*sec = bcdtoub (bcdsec);
	*min = bcdtoub (bcdmin);
	*hr = bcdtoub (bcdhr);
}


/*	Sets RTC date
	arguments:
		Year (0 - 9999), month (1 - 12), date (1 -31),
		day (1 - 7, Sunday = 1) as unsigned binary
	comments:
		Although the RTC is "Y2K" compliant, it may not calculate the
		absense of a leap day in 2100, 2200, etc. correctly.  So what?
*/
void RTCsetdate (UWORD mon, UWORD date, UWORD yr, UWORD day) {
	UBYTE bcdcent, bcdyr, bcdmon, bcddate, u, v;

	bcdcent = ubtobcd (yr / 100);
	bcdyr = ubtobcd (yr % 100);
	bcdmon = ubtobcd (mon);
	bcddate = ubtobcd (date);

	// wait for UIP bit == 0
	while (inportb (RTC_REGA) & 0x80);

	// set SET bit to inhibit update transfers
	u = inportb (RTC_REGB);
	u |= 0x80;
	outportb (RTC_REGB, u);

	outportb (RTC_DATE, bcddate);
	outportb (RTC_MON, bcdmon);
	outportb (RTC_YR, bcdyr);
	outportb (RTC_DAY, day);

	// select bank 1 for century register
	v = inportb (RTC_REGA);
	v |= 0x10;				// DV0 = 1
	outportb (RTC_REGA, v);

	outportb (RTC_CENT, bcdcent);

	// restore bank 0
	v &= ~0x10;				// DV0 = 0
	outportb (RTC_REGA, v);

	// clear SET bit to allow update transfers
	u &= ~0x80;
	outportb (RTC_REGB, u);
}


/*	Sets RTC date
	arguments:  Pointers to:
		Year (0 - 9999), month (1 - 12), date (1 -31),
		day (1 - 7, Sunday = 1) as unsigned binary
	comments:
*/
void RTCgetdate (UWORD *mon, UWORD *date, UWORD *yr, UWORD *day) {
	UBYTE bcdcent, bcdyr, bcdmon, bcddate, v;

	// wait for UIP bit == 0
	while (inportb (RTC_REGA) & 0x80);

	// select bank 1 for century register
	v = inportb (RTC_REGA);
	v |= 0x10;				// DV0 = 1
	outportb (RTC_REGA, v);

	bcdcent = inportb (RTC_CENT);

	// restore bank 0
	v &= ~0x10;				// DV0 = 0
	outportb (RTC_REGA, v);

	bcddate = inportb (RTC_DATE);
	bcdmon = inportb (RTC_MON);
	bcdyr = inportb (RTC_YR);
	*day = inportb (RTC_DAY);

	*mon = bcdtoub (bcdmon);
	*date = bcdtoub (bcddate);
	*yr = bcdtoub (bcdyr) + bcdtoub (bcdcent) * 100;
}


/*	Tests RTC RAM
*/
BOOLEAN RTCRAMtest (void) {
	WORD i, j;
	BOOLEAN faulty = FALSE;

	for (i = 14; i < 128; i++) {
		for (j = 0; j < 256; j++) {
			outportb (RTC_BASE + i, j);
			if (inportb (RTC_BASE + i) != j) {
				faulty = TRUE;
			}
		}
	}
	if (faulty)
		return FALSE;
	else
		return TRUE;
}


/*	Set RTC alarm time
		arguments:  (all in unsigned binary)
			1) hour (0 - 23)
			2) minute (0 - 59)
			3) second (0 - 59)
		comments:
			Does not turn the alarm on.  Use RTCalarmcontrol() for that.
			Out of range arguments will result in improper operation.
*/
void RTCsetalarmtime (UWORD hr, UWORD min, UWORD sec) {
	UBYTE bcdhr, bcdmin, bcdsec, u, v;

	bcdhr = ubtobcd (hr);
	bcdmin = ubtobcd (min);
	bcdsec = ubtobcd (sec);

	// wait for UIP bit == 0
	while (inportb (RTC_REGA) & 0x80);

	// set SET bit to inhibit update transfers
	u = inportb (RTC_REGB);
	u |= 0x80;
	outportb (RTC_REGB, u);

	outportb (RTC_ALARM_SEC, bcdsec);
	outportb (RTC_ALARM_MIN, bcdmin);
	outportb (RTC_ALARM_HR, bcdhr);

	// access extended register bank for alarm date register
	v = inportb (RTC_REGA);
	v |= 0x10;				// set DV0 bit
	outportb (RTC_REGA, v);

	// set alarm date register to "don't care".
	// you may follow this function with a call to RTCsetalarmdate()
	outportb (RTC_ALARM_DATE, 0xff);

	// restore bank 0
	v &= ~0x10;				// DV0 = 0
	outportb (RTC_REGA, v);

	// clear SET bit to allow update transfers
	u &= ~0x80;
	outportb (RTC_REGB, u);
}


/*	Set RTC alarm date
		arguments:  (all in unsigned binary)
			1) date of month (1 - 31)
		comments:
			Does not turn the alarm on.  Use RTCalarmcontrol() for that.
			Out of range arguments will result in improper operation.
			You don't have to specify an alarm date if you want the alarm
			to be generated at the same time each day.
*/
void RTCsetalarmdate (UWORD date) {
	UBYTE bcddate, u, v;

	bcddate = ubtobcd (date);

	// wait for UIP bit == 0
	while (inportb (RTC_REGA) & 0x80);

	// set SET bit to inhibit update transfers
	u = inportb (RTC_REGB);
	u |= 0x80;
	outportb (RTC_REGB, u);

	// access extended register bank for alarm date register
	v = inportb (RTC_REGA);
	v |= 0x10;				// set DV0 bit
	outportb (RTC_REGA, v);

	outportb (RTC_ALARM_DATE, bcddate);

	// restore bank 0
	v &= ~0x10;				// DV0 = 0
	outportb (RTC_REGA, v);

	// clear SET bit to allow update transfers
	u &= ~0x80;
	outportb (RTC_REGB, u);
}


/*	RTC alarm and interrupt control
		arguments:  (0 = disabled, non-zero = enabled)
			1) periodic interrupt enable
			2) alarm interrupt enable
			3) wake up (power-up) and kickstart enable
		comments:  you must program RTC with the alarm time and
			(optionally) date before turning alarm on.
			If wake-up is enabled, kickstart is also enabled.
*/
void RTCalarmcontrol (BOOLEAN perenable, BOOLEAN irqenable,
	BOOLEAN wakeenable) {

	UBYTE u, v, w;

	// read status flags to clear them
	inportb (RTC_REGC);

	// wait for UIP bit == 0
	while (inportb (RTC_REGA) & 0x80);

	// set SET bit to inhibit update transfers
	u = inportb (RTC_REGB);
	u |= 0x80;
	outportb (RTC_REGB, u);

	// program AIE bit
	u &= ~0x20;			// zero AIE bit
	u |= irqenable ? 0x20 : 0;

	// program PIE bit
	u &= ~0x40;			// zero PIE bit
	u |= perenable ? 0x40 : 0;

	// access extended register bank for alarm date register
	v = inportb (RTC_REGA);
	v |= 0x10;				// set DV0 bit
	outportb (RTC_REGA, v);

	// write WF, KF status flags to clear them
	w = inportb (RTC_REG4A);
	w &= ~0x03;
	outportb (RTC_REG4A, w);

	w = inportb (RTC_REG4B);
	w &= ~0x03;				// zero WIE and KSE bits
	w |= wakeenable ? 0x03 : 0;
	outportb (RTC_REG4B, w);

	// restore bank 0
	v &= ~0x10;				// DV0 = 0
	outportb (RTC_REGA, v);

	// clear SET bit to allow update transfers
	u &= ~0x80;
	outportb (RTC_REGB, u);
}


/*	Call this function to power down the DOS Stamp via the RTC
		comments:  before calling this function, set the desired power-up
			time with RTCsetalarmtime() and (optionally) RTCsetalarmdate().
			Then call RTCalarmcontrol with arguments of (FALSE, TRUE).
			You must have special power control circuitry connected to the
			-PWRUP pin to use this feature (see manual for details).
*/
void RTCpowerdown (void) {
	UBYTE v, w;

	// access extended register bank for PAB bit
	v = inportb (RTC_REGA);
	v |= 0x10;				// set DV0 bit
	outportb (RTC_REGA, v);

	// write PAB bit to set it
	w = inportb (RTC_REG4A);
	w |= 0x08;
	outportb (RTC_REG4A, w);

	// restore bank 0
	v &= ~0x10;				// DV0 = 0
	outportb (RTC_REGA, v);
}


/*	Sets the periodic interrupt rate of the RTC
		arguments:  the divisor (0 - 15)
		comments:  see the dsrtc.h include file for constant definitions for
			divisor argument
*/
void RTCsetperiod (UBYTE p) {
	UBYTE u;

	u = inportb (RTC_REGA);
	u &= ~0x0f;				// zero RS[3..0] bits
	u |= p & 0x0f;			// set divisor bits
	outportb (RTC_REGA, u);
}
